package bg.bas.dcl.MWE;

import java.util.ArrayList;

public class MWEComponent extends ArrayList<MWEComponent>{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	public String lemma = "";
	public String gram = "";
	
	public String flex = "";
	public String flexcons = "";
	
	public String head = "";
	public String struct = "";
	
	public String modifierstype = "";
	public String connectiontype = "";
	
	public boolean full = false;
	public boolean independent = true;
	
	public ArrayList<MWEJoint> joints = new ArrayList<MWEJoint>();
	
	public MWEComponent(){
		
	}
	
	public String toString(){
		String res = "";
		
		if (!independent) return "";
		
		for (MWEComponent comp : this) {
			res += comp.toString() +" ";
		}
		
		
		res += lemma;
		
		if (this.size()>0) res = "( " + res + " ) ";

		res += "<"+gram+">";
		
		if (!flex.equals("")) {
			res += flex;
			if (!flexcons.equals("")) res += ","+flexcons;
		}
			
		return res;
	}
	
	public String reducedMWEDescription(){
		String res = "";
		
		defineJoints();
		
//		if (!independent) return "";
		
		for (int i = 0; i<size(); i++) {
			MWEComponent comp = get(i);
			
			res += comp.reducedMWEDescription() +" ";
			if (i<size()-1) res += "=" + joints.get(i).type +"= ";
		}

		res += "<";
		if (!independent) res += gram.substring(0, 1);
		else res += gram;
		
		if (!flex.equals("")) {
			res += "_"+flex;
			if (!flexcons.equals("")) res += ","+flexcons;
		}

		res += ">";
		
		if (this.size()>0) res = "( " + res + " ) ";


		return res;
	}


	public String fullMWEDescription(){
		String res = "";
		
		res += lemma ;
		res += "\t" + struct ;
		res += "\t" + head ;
			
		return res;
	}
	
	public String componentsDescription(){
		String res = "";
		
		for (MWEComponent c : this) {
			res += c.gram + "<" +c.struct+">" +" ";
		}
			
		return res;
	}

	public void defineJoints(){
		joints = new ArrayList<MWEJoint>();
		
		int intno = 1;
		
		for (int i = 0; i<size()-1; i++){
			MWEComponent c1 = get(i);
			MWEComponent c2 = get(i+1);
			
			for (int j = 0; j<c1.size()-1; j++){
				MWEComponent cc1 = c1.get(j);
				MWEComponent cc2 = c1.get(j+1);
				
				MWEJoint jjoint = new MWEJoint();
				
				jjoint.number = intno++;
				
				if (cc1.gram.startsWith("A") && cc2.gram.startsWith("N")) {
					jjoint.type = "A-N";
					jjoint.insertions = "InsCl";
					jjoint.modifications = "PreA";
				}else
				if (cc1.gram.startsWith("P") && cc2.gram.startsWith("A")) {
					jjoint.type = "P-A";
					jjoint.insertions = "NoIns";
					jjoint.modifications = "PreD";
				}else
				if (cc1.gram.startsWith("P") && cc2.gram.startsWith("N")) {
					jjoint.type = "P-N";
					jjoint.insertions = "NoIns";
					jjoint.modifications = "PreAD";
				}else
				if (cc1.gram.startsWith("D") && cc2.gram.startsWith("A")) {
					jjoint.type = "D-A";
					jjoint.insertions = "InsCl";
					jjoint.modifications = "PreA";
				}else
				if (cc1.gram.startsWith("P") && cc2.gram.startsWith("P")) {
					jjoint.type = "P-P";
					jjoint.insertions = "NoIns";
					jjoint.modifications = "NoMod";
				}else
				if (cc1.gram.startsWith("P") && cc2.gram.startsWith("C")) {
					jjoint.type = "P-C";
					jjoint.insertions = "NoIns";
					jjoint.modifications = "NoMod";
				}else {
					jjoint.type = cc1.gram.substring(0, 1)+"-"+cc2.gram.substring(0, 1);
					jjoint.insertions = "NoIns";
					jjoint.modifications = "NoMod";
				}
				
				joints.add(jjoint);

			}

			
			MWEJoint joint = new MWEJoint();
			
			joint.number = intno++;
			

			if (c1.gram.startsWith("V") && c2.gram.startsWith("NP")) {
				joint.type = "V-NP";
				joint.insertions = "InsAll";
				joint.modifications = "ModPreADPostDPP";
			}else
			if (c2.gram.startsWith("DP")) {
				joint.type = "V-DP";
				joint.insertions = "InsAll";
				joint.modifications = "ModPreDPostDPP";
			}else
			if (c1.gram.startsWith("V") && c2.gram.startsWith("PP")) {
				joint.type = "V-PP";
				joint.insertions = "InsAll";
				joint.modifications = "ModPreDPostDPP";
			}else
			if (c1.gram.startsWith("NP") && c2.gram.startsWith("PP")) {
				joint.type = "NP-PP";
				joint.insertions = "InsAll";
				joint.modifications = "ModPostD";
			}else
			if (c1.gram.startsWith("P") && c2.gram.startsWith("P")) {
				joint.type = "PP-PP";
				joint.insertions = "InsAll";
				joint.modifications = "NoMod";
			}else
			if (c1.gram.startsWith("P") && c2.gram.startsWith("C")) {
				joint.type = "PP-CP";
				joint.insertions = "NoIns";
				joint.modifications = "NoMod";
			}else
			if (c1.gram.startsWith("V") && c2.gram.startsWith("T")) {
				joint.type = "V-T";
				joint.insertions = "InsCl";
				joint.modifications = "NoMod";
			}else
			if (c1.gram.startsWith("T") && c2.gram.startsWith("V")) {
				joint.type = "T-V";
				joint.insertions = "InsCl";
				joint.modifications = "NoMod";
			}else
			if (c1.gram.startsWith("C") && c2.gram.startsWith("V")) {
				joint.type = "C-V";
				joint.insertions = "NoIns";
				joint.modifications = "NoMod";
			}else {
				joint.type = (c1.gram.length()>0?c1.gram.substring(0, 1):"")+"-"
										+(c2.gram.length()>0?c2.gram.substring(0, 1):"");
				joint.insertions = "NoIns";
				joint.modifications = "NoMod";
			}
			
			joints.add(joint);

		}
		
		if (size() == 0) return;
		
		MWEComponent c2 = get(size()-1);
		
		
		for (int j = 0; j<c2.size()-1; j++){
			MWEComponent cc1 = c2.get(j);
			MWEComponent cc2 = c2.get(j+1);
			
			MWEJoint jjoint = new MWEJoint();
			
			jjoint.number = intno++;
			
			if (cc1.gram.startsWith("A") && cc2.gram.startsWith("N")) {
				jjoint.type = "A-N";
				jjoint.insertions = "InsCl";
				jjoint.modifications = "ModPreA";
			}else
			if (cc1.gram.startsWith("P") && cc2.gram.startsWith("A")) {
				jjoint.type = "P-A";
				jjoint.insertions = "NoIns";
				jjoint.modifications = "ModPreD";
			}else
			if (cc1.gram.startsWith("P") && cc2.gram.startsWith("N")) {
				jjoint.type = "P-N";
				jjoint.insertions = "NoIns";
				jjoint.modifications = "ModPreAD";
		}else
			if (cc1.gram.startsWith("D") && cc2.gram.startsWith("A")) {
				jjoint.type = "D-A";
				jjoint.insertions = "NoIns";
				jjoint.modifications = "NoMod";
			}else {
				jjoint.type = cc1.gram.substring(0, 1)+"-"+cc2.gram.substring(0, 1);
				jjoint.insertions = "NoIns";
				jjoint.modifications = "NoMod";
		}

			joints.add(jjoint);

		}

	}
	
	public String jointsDescription(){
		String res = "";
		
		defineJoints();
		
		for (MWEJoint jj : joints) {
			res += jj.number+","+jj.insertions+","+jj.modifications +" ";
		}
			
		return res;
	}

}
