package bg.bas.dcl.MWE;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.HashMap;
import java.util.Scanner;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;

import bg.bas.dcl.general.TextProcessor;
import bg.bas.dcl.inflection.InitialParametersReader;

public class MWEProcessor {
	
	HashMap<String,String> flextypes = new HashMap<String,String>();
	int ID = 0;
	
	public MWEProcessor(){
		
	}
	
	public MWEProcessor(String noojdict){
		flextypes = InitialParametersReader
				.readDictionaryFromFile(noojdict,0,1);
		
	}
	
	public void processMWElist(String infile, String outfile){
		
		try{
			
				
			Writer out = new OutputStreamWriter(
					new FileOutputStream(outfile));
			
			Scanner s =  new Scanner(new File (infile));
			
			while (s.hasNextLine()){
				
				String l = s.nextLine();
				
				System.out.println(l);
				
				MWEComponent com = getSyntacticTypeMWE(l);
				if (com == null) continue;
				String res = com.toString();
				
				out.write(l+"\n");
				out.write(res.trim()+"\n");
				out.write(com.fullMWEDescription()+"\n");
				out.write(com.reducedMWEDescription()+"\n");
				out.write("\t"+ com.componentsDescription()+"\n");
				
				out.write("\t"+ com.jointsDescription()+"\n");
				
				out.flush();
			}
			
			s.close();
			
			out.flush();
			out.close();
		}catch(Exception e){
			e.printStackTrace();
		}
		
	}

	
	public void processMWElistIntoJSON(String infile, String outdir){
		
		try{
			
			
			Scanner s =  new Scanner(new File (infile));
			
			while (s.hasNextLine()){
				
				String l = s.nextLine();
				
				System.out.println(l);
				
				MWEComponent com = getSyntacticTypeMWE(l);
				if (com == null) continue;
				
				
				String fname = ""+(ID++);
				while (fname.length()<8) fname ="0"+fname;
				Writer out = new OutputStreamWriter(
					new FileOutputStream(outdir + fname + ".json"));
			
				JSONObject obj = getJSONObjectMWE(l);

				
				out.write(obj.toJSONString());
				
				out.flush();
				out.close();
			}
			
			s.close();
			
			
		}catch(Exception e){
			e.printStackTrace();
		}
		
	}
	
	
	public JSONObject getJSONObjectMWE(String l){
		JSONObject obj = new JSONObject();
		
		MWEComponent mwe = getSyntacticTypeMWE(l);
		
		obj.put("lemma", mwe.lemma);
		obj.put("gram", mwe.gram);
		obj.put("synttype", mwe.struct);
		obj.put("semtype", "");
		
		JSONArray jsonlist = new JSONArray();

		for (MWEComponent mc : mwe){
			
			JSONObject cobj = new JSONObject();
			
			cobj.put("gram", mc.gram);
			cobj.put("synttype", mc.struct);
			cobj.put("flecttype", mc.flex);
			cobj.put("lemma", mc.lemma);

			
			jsonlist.add(cobj);
			
		}
		
		obj.put("components", jsonlist);
		
		return obj;
	}

    public MWEComponent getSyntacticTypeMWE(String line){
    	
    	String[] words = line.split(" ");
    	
    	MWEComponent mwe = new MWEComponent();
    	MWEComponent currcomp = new MWEComponent();
    	
cc:    	for (int i = 0; i<words.length; i++) {
    		
    		String word = words[i];
    		
    		String[] data = word.split("/");
    		
    		String wordform = data[0];
    		String lemma = data[1];
    		String pos = data[2];
    		
    		mwe.lemma += wordform +" ";
    		
    		if (wordform.equals("аз")) return null;
    		
 //   		if (pos.startsWith("C")) return null;
    		
    		
    		if (lemma.equals("се")
    				|| lemma.equals("си")) {
    			
    				
    			if (currcomp.size()==0 
    					|| !currcomp.get(currcomp.size()-1).lemma.equals("себе")) {
    				
					MWEComponent currcompcomp = new MWEComponent();
	    			currcompcomp.struct += "_se";

	    			currcompcomp.lemma = lemma;
	    			currcompcomp.gram = pos;
	    			
	    			
	    			boolean added = false;
	    			
	    			for (int j = 0; j<mwe.size(); j++) {
	    				if (mwe.get(j).gram.startsWith("V")) {
		         			
		        			mwe.add(j+1,currcompcomp);
		         			added = true;
		         			break;
		    			}
	    			}
	
	    			if (!added) mwe.add(currcompcomp);
	    			
	    			
	    			mwe.head += "_" +(lemma.equals("се")?"se":"si");
	    			
	    			
    			} else {
					MWEComponent currcompcomp = new MWEComponent();
	    			currcompcomp.struct += "_se";

	    			currcompcomp.lemma = lemma;
	    			currcompcomp.gram = pos;
	    			mwe.add(currcompcomp);
    			}
    			
    			continue cc;
    		}
    		
    		if (pos.startsWith("V") && pos.contains("3")) return null;
    		
    		if (pos.startsWith("V") && pos.contains("q")) {
    			pos = "A"+pos.substring(pos.indexOf("q")+1);
    		}
    		
    		if (pos.startsWith("B")) {
    			pos = "A"+pos.substring(1);
    		}

     		
    		if (pos.startsWith("V") && !pos.contains("q")) {
    			
    			currcomp.lemma = lemma;
    			currcomp.full = true;
    			
    			if (lemma.equals("съм")
    					&& ((mwe.size()>0 && mwe.get(0).gram.startsWith("V"))
    							|| (mwe.size()>1 && mwe.get(1).gram.startsWith("V")))){
    				continue;					
    			}
    			
    			currcomp.gram += "V_";
    			
    			if (pos.contains("L")) currcomp.gram += "L"; //lichen
    			else if (pos.contains("B")) currcomp.gram += "B";//3-lichen
    			else currcomp.gram += "U";//unknown

    			if (pos.contains("I")) currcomp.gram += "I";//nesv.vid
    			else if (pos.contains("P")) currcomp.gram += "P";//sv.vid
    			else currcomp.gram += "U";//unknown
    			
    			if (pos.contains("T")) currcomp.gram += "T";//transitive
    			else if (pos.contains("N")) currcomp.gram += "N";//non-trans
    			else currcomp.gram += "U";//unknown
    			
				if (flextypes.containsKey(lemma))
					currcomp.flex = flextypes.get(lemma);
				else
					currcomp.flex = "UNKNOWN";
	
    			//currcomp.flexcons = "NO_PASS";
				
    			if (!lemma.equals("съм") && mwe.size()>0 && mwe.get(0).lemma.equals("съм")){
    				mwe.remove(0);
    				
    			}
    			if (!lemma.equals("съм") && mwe.size()>1 && mwe.get(1).lemma.equals("съм")){
    				mwe.remove(1);
    								
    			}
    			
    			

				if (mwe.size()>0 && (mwe.get(mwe.size()-1).lemma.equals("да")
							|| mwe.get(mwe.size()-1).lemma.equals("си")
							|| mwe.get(mwe.size()-1).lemma.equals("се"))) {
					
					
					mwe.add(currcomp);
					
					if (mwe.get(0).lemma.equals("си")
							|| mwe.get(0).lemma.equals("се")){
						mwe.add(mwe.get(0));
						mwe.remove(0);
					}
						
					
				}
				else mwe.add(0,currcomp);
				
				if (!mwe.struct.startsWith("V")) mwe.struct = "V " + mwe.struct;
    			mwe.head = currcomp.gram + mwe.head;
    			
				if (mwe.size()>0 && mwe.get(mwe.size()-1).lemma.equals("не")){
					mwe.add(0,mwe.get(mwe.size()-1));
					mwe.remove(mwe.size()-1);
					
	    			mwe.head += "_neg";

				}

    			currcomp = new MWEComponent();
    		}
    		
    		
    		if (pos.startsWith("R")) {
    			
    			MWEComponent currcompcomp = new MWEComponent();
    			
    			currcompcomp.lemma = lemma;
    			currcompcomp.gram = "P";
    			
    			
    			currcomp.add(currcompcomp);
    			
    			currcomp.lemma += wordform+" ";
    			
    			currcomp.struct += "P ";

    			if (i < words.length-1){
    				String[] data1 = words[i+1].split("/");
    	    		
    	    		String wordform1 = data1[0];
    	    		String lemma1 = data1[1];
    	    		String pos1 = data1[2];
    	    		
    	    		if (lemma1.startsWith("кой")
    	    				&& lemma1.equals("да")){
    	    			
    	    			mwe.add(currcomp);
    	    			mwe.add(currcompcomp);
    	    			currcomp = new MWEComponent();

    	    		} else {
    	    			currcomp.gram += "PP_";
    	    		}
    			}else{
        			currcomp.gram += "PP_";

    			}

    		
     		}

    		
    		
    		if (pos.startsWith("A")) {
    			
    			MWEComponent currcompcomp = new MWEComponent();
    			
    			currcompcomp.lemma = lemma;
    			currcompcomp.gram = pos;
    			
    			currcompcomp.independent = false;
    			currcomp.add(currcompcomp);
    			
    			currcomp.lemma += wordform+" ";
 
    			currcomp.struct += "A ";
    	   		
    			if (i < words.length-1){
    				String[] data1 = words[i+1].split("/");
    	    		
    	    		String wordform1 = data1[0];
    	    		String lemma1 = data1[1];
    	    		String pos1 = data1[2];
    	    		
    	    		if (!pos1.startsWith("N")){
    	    			
    	    			currcomp.gram = "AP_";
    	    			
    	    			mwe.add(currcomp);
    	    			currcomp = new MWEComponent();

    	    		}
    			}else{
        			currcomp.gram = "AP_";

    	   			mwe.add(currcomp);
        			currcomp = new MWEComponent();

    			}
    		
     		}

    		if (pos.startsWith("P")) {
    			
    			MWEComponent currcompcomp = new MWEComponent();   			
    			    			
    			currcompcomp.lemma = lemma;
    			currcompcomp.gram = pos;
    			
    			currcompcomp.independent = false;
    			currcomp.add(currcompcomp);
    			
    			currcomp.lemma += wordform+" ";
    			currcomp.struct += "A ";
    			
    			if (i < words.length-1){
    				String[] data1 = words[i+1].split("/");
    	    		
    	    		String wordform1 = data1[0];
    	    		String lemma1 = data1[1];
    	    		String pos1 = data1[2];
    	    		
    	    		if (!pos1.startsWith("N")&& !lemma.equalsIgnoreCase("себе")){
    	       			mwe.add(currcomp);
    	    			currcomp = new MWEComponent();

    	    		}
    			}else{
    	   			mwe.add(currcomp);
        			currcomp = new MWEComponent();

    			}
    		
     		}

   		if (pos.startsWith("D")
   				|| pos.startsWith("I")) {
    			
    			MWEComponent currcompcomp = new MWEComponent();
    			
    			currcompcomp.lemma = lemma;
    			currcompcomp.gram = pos;
    			
    			currcompcomp.independent = false;
    			currcomp.add(currcompcomp);
    			
    			currcomp.struct += pos.substring(0,1)+" ";
    			
    			currcomp.lemma += wordform+" ";
    			currcomp.gram += pos+" ";
    			
    			if (!currcomp.gram.startsWith("PP")) mwe.struct += " DP";
    		
    			mwe.add(currcomp);
    			currcomp = new MWEComponent();

     		}

   		if (pos.startsWith("T")
   				|| pos.startsWith("C")) {
			
			currcomp.lemma = lemma;
			currcomp.gram = pos;
			
			currcomp.independent = true;
			currcomp.struct += pos.substring(0,1)+" ";
			
			mwe.add(currcomp);
			currcomp = new MWEComponent();
			

 		}

  	
    		if (pos.startsWith("N")) {
    			
    			MWEComponent currcompcomp = new MWEComponent();
    			
    			currcompcomp.lemma = lemma;
    			currcompcomp.gram += pos;
    			
     			currcomp.gram += "NP_N";
    			
    			if (pos.contains("C")) currcomp.gram += "C";
    			else if (pos.contains("H")) currcomp.gram += "H";
    			else currcomp.gram += "U";
    			
    			if (pos.contains("S")) currcomp.gram += "S";
    			else if (pos.contains("P")) currcomp.gram += "P";
    			else if (pos.contains("s")) currcomp.gram += "S"; 
    			else if (pos.contains("p")) currcomp.gram += "P";
    			else currcomp.gram += "O";
    			
    			if (pos.contains("M")) currcomp.gram += "M";
    			else if (pos.contains("F")) currcomp.gram += "F";
    			else if (pos.substring(1).contains("N")) currcomp.gram += "N";
    			else currcomp.gram += "U";
    			
       			if (pos.contains("d")) currcomp.gram += "D";
    			else if (pos.contains("o")) currcomp.gram += "I";
    			else if (pos.contains("h")) currcomp.gram += "D"; 
    			else currcomp.gram += "A";

        		if (pos.contains("N") && pos.contains("l")) return null;

    			
    			currcompcomp.flex = "";
    			currcompcomp.flexcons = "";
    			currcompcomp.independent = false;
    			
    			currcomp.add(currcompcomp);

    			currcomp.struct += "N ";
    			
    			currcomp.lemma += wordform+" ";
    			
    			currcomp.full = true;
    			
    			currcomp.flex = currcomp.gram;
 
    			mwe.add(currcomp);
    			
    			if (!currcomp.gram.startsWith("PP")) mwe.struct += " NP";
    			currcomp = new MWEComponent();
    		}

    	}
    	
    	return mwe;
    	
    }



	public void extractVerbsAndCheckFlex(String file, String noojdict){
		
		try{
			
			HashMap<String,String> flextypes = new HashMap<String,String>();
			HashMap<String,Integer> lemmafreq = new HashMap<String,Integer>();
			flextypes = InitialParametersReader
					.readDictionaryFromFile(noojdict,0,1);
			
			Scanner s =  new Scanner(new File (file));
			
			while (s.hasNextLine()){
				
				String l = s.nextLine();
				if (l.startsWith("<")) continue;
				if (!l.contains("pos=\"V")) continue;
				
				String lemma = l.substring(l.indexOf("lemma")+7);
				lemma = lemma.substring(0, lemma.indexOf("\""));
				
				
				
				if (lemmafreq.containsKey(lemma)) lemmafreq.put(lemma,lemmafreq.get(lemma)+1);
				else lemmafreq.put(lemma,1);
				
			}
			
			for (String lemma : lemmafreq.keySet()){
				if (flextypes.containsKey(lemma))
					System.out.println(lemma+"\t"+lemmafreq.get(lemma)+"\t"+flextypes.get(lemma));
				else
					System.out.println(lemma+"\t"+lemmafreq.get(lemma)+"\t"+false);
			}
			
			s.close();
		}catch(Exception e){
			e.printStackTrace();
		}
		
	}
	
	public void prepareMWElist(String infile, String noojdict, String outfile){
		
		try{
			
			HashMap<String,String> flextypes = new HashMap<String,String>();

			flextypes = InitialParametersReader
					.readDictionaryFromFile(noojdict,0,1);
			
			Writer out = new OutputStreamWriter(
					new FileOutputStream(outfile));
			
			Scanner s =  new Scanner(new File (infile));
			
			while (s.hasNextLine()){
				
				String l = s.nextLine();
				if (!l.startsWith("<mwe")) continue;
				if (!l.contains("pos=\"V")) continue;
				
				String lemma = l.substring(l.indexOf("lemma")+7);
				lemma = lemma.substring(0, lemma.indexOf("\""));
				
				String tagged = TextProcessor.applyServiceToText(lemma, 2);
				String[] words = tagged.split("\n");
				
				String res = "";
				
				for (String w : words) {
					String[] data = w.split("\t");
					if (data.length < 4) break;
					
					res += data[0] 
							+ "/" + data[data.length-2] 
							+ "/" + data[data.length-1];
					

					res += " ";
					
				}
				
				out.write(res.trim()+"\n");
				out.flush();
			}
			
			s.close();
			
			out.flush();
			out.close();
		}catch(Exception e){
			e.printStackTrace();
		}
		
	}


}
