package bg.bas.dcl.inflection;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Scanner;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


public class FileHandler {
	
	
	public FileHandler(){
		
	}
	
	  /**
	   * Recursively walk a directory tree and return a List of all
	   * Files found; the List is sorted using File.compareTo().
	   *
	   * @param aStartingDir is a valid directory, which can be read.
	   */
	   public static List<File> getFileListing(
	     File aStartingDir
	   ) throws FileNotFoundException {
	     validateDirectory(aStartingDir);
	     List<File> result = getFileListingNoSort(aStartingDir);
	     Collections.sort(result);
	     return result;
	   }

	   // PRIVATE //
	   private static List<File> getFileListingNoSort(
	     File aStartingDir
	   ) throws FileNotFoundException {
	     List<File> result = new ArrayList<File>();
	     File[] filesAndDirs = aStartingDir.listFiles();
	     List<File> filesDirs = Arrays.asList(filesAndDirs);
	     for(File file : filesDirs) {
	       result.add(file); //always add, even if directory
	       if ( ! file.isFile() ) {
	         //must be a directory
	         //recursive call!
	         List<File> deeperList = getFileListingNoSort(file);
	         result.addAll(deeperList);
	       }
	     }
	     return result;
	   }

	   /**
	   * Directory is valid if it exists, does not represent a file, and can be read.
	   */
	   private static void validateDirectory (
	     File aDirectory
	   ) throws FileNotFoundException {
	     if (aDirectory == null) {
	       throw new IllegalArgumentException("Directory should not be null.");
	     }
	     if (!aDirectory.exists()) {
	       throw new FileNotFoundException("Directory does not exist: " + aDirectory);
	     }
	     if (!aDirectory.isDirectory()) {
	       throw new IllegalArgumentException("Is not a directory: " + aDirectory);
	     }
	     if (!aDirectory.canRead()) {
	       throw new IllegalArgumentException("Directory cannot be read: " + aDirectory);
	     }
	   }

		public static void makeParent(String path,String mainPath){
			  String parent = (new File(path)).getParent();
			 
			  if ((new File(parent)).exists()) return;
			  
			  if (!parent.equals(mainPath)) {
				  makeParent(parent, mainPath);
			  } 
			  
			  (new File(parent)).mkdir();
			  
		}

		public void saveToFile(String text, String encoding, String filename){

			String path = "/";
			
		    try{

		    
				path = path.concat(filename);
		    
				
		      
				System.out.println (path);
//				if ((new File(path)).exists()) return;

				makeParent(path,"/");

				Writer out = new OutputStreamWriter(new FileOutputStream(path),encoding);
		      
		 	    out.write(text);
		 	    out.flush();
		 	    out.close();
	 	    

		    } catch(Exception e) {
		    	System.err.println(e.getMessage());  
		    }
			
		}

		public static void copyFile(File from, File to) throws IOException {

		    makeParent(to.getAbsolutePath(),"/");

			if (!to.exists()) {
				to.createNewFile();
			}

			try {
				FileChannel in = new FileInputStream(from).getChannel();
				FileChannel out = new FileOutputStream(to).getChannel();

				out.transferFrom(in, 0, in.size());
				
				in.close();
				out.close();
			} catch (Exception e) {
			}
		}

		public static void copyExcerpt(File from, File to, int excerptSize, int distanceSize) throws IOException {

		    makeParent(to.getAbsolutePath(),"/");

			if (!to.exists()) {
				to.createNewFile();
			}

			try {
				Scanner in = new Scanner(from);
				Writer out = new OutputStreamWriter(
						new FileOutputStream(to));

				int size = 0;
				boolean write = true;
				
				while (in.hasNextLine()){
					String line = in.nextLine();
					size += line.length();
					
					if (write) {
						out.write(line + "\n");
						out.flush();
						
						if (size > excerptSize) {
							size = 0;
							write = false;
						}
					} else {
						
						if (size > distanceSize) {
							size = 0;
							write = true;
						}
					}

				}

				
				in.close();
				
				out.flush();
				out.close();
			} catch (Exception e) {
			}
		}
		

		public static boolean deleteDirectory(File dir) {
		    if(! dir.exists() || !dir.isDirectory())    {
		        return false;
		    }

		    String[] files = dir.list();
		    for(int i = 0, len = files.length; i < len; i++)    {
		        File f = new File(dir, files[i]);
		        if(f.isDirectory()) {
		            deleteDirectory(f);
		        }else   {
		            f.delete();
		        }
		    }
		    return dir.delete();
		}

}
