package bg.bas.dcl.semantic.lexical_db;

import java.io.File;
import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Scanner;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class BulNet extends HashMap<String, Synset> {
	
	Document doc;
	HashMap<String,ArrayList<String>> index = 
			new HashMap<String,ArrayList<String>>();
	HashMap<String,ArrayList<String>> hyponymIndex = 
			new HashMap<String,ArrayList<String>>();
	HashMap<String,ArrayList<String>> hyponymInstancesIndex = 
			new HashMap<String,ArrayList<String>>();

	
	public String lang = "bg";
	
	public BulNet(String xmlFile){
		initialiseFromXML(xmlFile);	 
	
		createIndex();
	
}
	
	public BulNet(){
		
	}
	
	public void createIndex(){
		
		for (String id : keySet()) {
			
			Synset s = get(id);
			
			for (Literal l : s.literals){
				ArrayList<String> ind = new ArrayList<String> ();
				if (l.lemma.length()>0){
					if (index.containsKey(l.lemma)) {
						ind = index.get(l.lemma);
					}
					ind.add(id);
					index.put(l.lemma, ind);
				}else{
					if (index.containsKey(l.word)) {
						ind = index.get(l.word);
					}
					ind.add(id);
					index.put(l.word, ind);

				}
			}
			
			
			for (Relation r : s.relations) {
				if (r.type.equalsIgnoreCase("hypernym")) {
					ArrayList<String> ind = new ArrayList<String> ();
					if (hyponymIndex.containsKey(r.ID)) {
						ind = hyponymIndex.get(r.ID);
					}
					ind.add(id);
					hyponymIndex.put(r.ID, ind);


//					System.out.println("SS: "+r.ID + " "+r.type);
				}
				
				if (r.type.equalsIgnoreCase("instance_hypernym")) {
					ArrayList<String> ind = new ArrayList<String> ();
					if (hyponymInstancesIndex.containsKey(r.ID)) {
						ind = hyponymInstancesIndex.get(r.ID);
					}
					ind.add(id);
					hyponymInstancesIndex.put(r.ID, ind);


//					System.out.println("SS: "+r.ID + " "+r.type);
				}

			}
			
			
			
		}

		
		
	}
	
	public BulNet(String xmlFile,String MSfile){
	
	
			initialiseFromXML(xmlFile);	 
		
			assignMorphosemantic(MSfile);
			createIndex();
			System.out.println("Indexing... " + index.size());
	}
	
	
	public void initialiseFromXML(String xmlFile){
		

		try {

			System.out.println("Reading BulNet from XML...");
			
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			
			doc = dBuilder.parse(new File(xmlFile));
			
			//optional, but recommended
			//read this - http://stackoverflow.com/questions/13786607/normalization-in-dom-parsing-with-java-how-does-it-work
			doc.getDocumentElement().normalize();
			
			NodeList synsetList = doc.getElementsByTagName("SYNSET");
			
			for (int i = 0; i < synsetList.getLength(); i++) {
				 
				Node synsetNode = synsetList.item(i);
				
				if (synsetNode.getNodeType() != Node.ELEMENT_NODE) continue;
				
				Element element = (Element) synsetNode; 
				
				addSynsetFromElement(element);

			}

			System.out.println("BulNet read: " + this.size() + " synsets.");
		
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	

	}
	
	
	public void addSynsetFromElement(Element element){
		
		Synset synset = new Synset();
		
		String ID = getSynsetInfo(element, "ID");
		String pos = getSynsetInfo(element, "POS");
		if (pos.equals("r")) pos="b";
		if (ID.endsWith("-r")) ID = ID.replace("-r", "-b");
		
		String def = getSynsetInfo(element, "DEF");
		String stamp = getSynsetInfo(element, "STAMP");
		String status = getSynsetInfo(element, "STATUS");
		String semclass = getSynsetInfo(element, "SEM_CLASS");
		String bcs = getSynsetInfo(element, "BCS");
		
		String positivity = getSynsetInfo(element, "POSITIVITY");
		String negativity = getSynsetInfo(element, "NEGATIVITY");
		String snote = getSynsetInfo(element, "SNOTE");
		String domain = getSynsetInfo(element, "DOMAIN");

		
		ArrayList<Literal> literals = getSynsetSynonyms(element);
		ArrayList<Relation> relations = getSynsetRelations(element);

		synset.ID = ID;
		synset.stamp = stamp;
		synset.status = status;
		synset.pos = pos;
		synset.definition = def;
		synset.semclass = semclass;
		synset.bcs = Integer.valueOf(bcs);
		synset.snote = snote;
		synset.domain = domain;

		synset.literals = literals;
		synset.relations = relations;
	
		if (positivity.length()>0) synset.positivity = Double.valueOf(negativity);
		if (negativity.length()>0) synset.negativity = Double.valueOf(positivity);

		ArrayList<String> us = getSynsetUsages(element);
		synset.usages = us;
		

		put(ID, synset);
		
//		System.out.println(synset.toXML());
		
	}

	
	public String getSynsetInfo(Element element, String what){
		
		
		
		String result = "";
		
		if (element.hasAttribute(what)) {
			result = element.getAttribute(what);
		}
		if (element.hasAttribute(what.toLowerCase())) {
			result = element.getAttribute(what.toLowerCase());
		}
		
		if (!result.equals("")) return result;

		NodeList list = element.getElementsByTagName(what);
		
		for (int i = 0; i < list.getLength(); i++) {
			 
			Node node = list.item(i);
			
			if (node.getNodeType() != Node.ELEMENT_NODE) continue;
			
			Element e = (Element) node; 
			
			result = e.getTextContent();
			

		}


		return result;
		
	}

	
	public ArrayList<Literal> getSynsetSynonyms(Element element){
		
		ArrayList<Literal> result = new ArrayList<Literal>();
		
		NodeList list = element.getElementsByTagName("SYNONYM");
		
		for (int i = 0; i < list.getLength(); i++) {
			 
			Node node = list.item(i);
			
			if (node.getNodeType() != Node.ELEMENT_NODE) continue;
			
			Element e = (Element) node; 
			
			//
			NodeList list1 = e.getElementsByTagName("LITERAL");
			
			for (int j = 0; j < list1.getLength(); j++) {
				 
				Node node1 = list1.item(j);
				
				if (node1.getNodeType() != Node.ELEMENT_NODE) continue;
				
				Element e1 = (Element) node1; 
				
				Literal lit = new Literal();
				
				lit.word = e1.getChildNodes().item(0).getTextContent();
				
				lit.lemma = getSynsetInfo(e1, "LEMMA");
				lit.sense = getSynsetInfo(e1, "SENSE");
				lit.status = getSynsetInfo(e1, "STATUS");
				lit.lnote = new ArrayList<String>();
				
				NodeList listk = e1.getElementsByTagName("LR");
				
				for (int k = 0; k < listk.getLength(); k++) {
					 
					Node nodek = listk.item(k);
					
					if (nodek.getNodeType() != Node.ELEMENT_NODE) continue;
					
					Element ek = (Element) nodek; 
					
					Relation r = new Relation();
					
					r.ID = ek.getChildNodes().item(0).getTextContent();
					
					r.type = getSynsetInfo(ek, "TYPE");
					
					lit.relations.add(r);

				}

				NodeList listl = e1.getElementsByTagName("LNOTE");
				
				for (int l = 0; l < listl.getLength(); l++) {
					 
					Node nodel = listl.item(l);
					
					if (nodel.getNodeType() != Node.ELEMENT_NODE) continue;
					
					Element ek = (Element) nodel; 
					
					lit.lnote.add(ek.getTextContent());

				}


				
				//
				result.add(lit);

			}


		}


		return result;
		
	}

	public ArrayList<Relation> getSynsetRelations(Element element){
		
		ArrayList<Relation> result = new ArrayList<Relation>();
		
		NodeList list = element.getElementsByTagName("ILR");
		
		for (int i = 0; i < list.getLength(); i++) {
			 
			Node node = list.item(i);
			
			if (node.getNodeType() != Node.ELEMENT_NODE) continue;
			
			Element e = (Element) node; 
			
			Relation r = new Relation();
			
			r.ID = e.getChildNodes().item(0).getTextContent();
			r.type = getSynsetInfo(e, "TYPE");
			
			result.add(r);

		}
		

		return result;
		
	}
	
	public ArrayList<String> getSynsetUsages(Element element){
		
		ArrayList<String> result = new ArrayList<String>();
		
		NodeList list = element.getElementsByTagName("USAGE");
		
		for (int i = 0; i < list.getLength(); i++) {
			 
			Node node = list.item(i);
			
			if (node.getNodeType() != Node.ELEMENT_NODE) continue;
			
			Element e = (Element) node; 
			
			if (e != null
					&& e.getChildNodes().item(0) != null)
				result.add(e.getChildNodes().item(0).getTextContent());

		}


		return result;
		
	}


	
	public String toXML(){
		
		String result = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
		
		result += "<WORDNET lang=\""+lang+"\">\n";
		
		for (String s : keySet()){
			Synset syn = get(s);
			result += syn.
					toXML() + "\n";
//			System.out.println(syn.toXML());
		}

		result += "</WORDNET>\n";

		return result;

		
	}

	public void saveAsXML(Writer out){
		
		try{
			out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
			
			out.write("<WORDNET lang=\""+lang+"\">\n");
			
			for (String s : keySet()){
				Synset syn = get(s);
				out.write(syn.toXML() + "\n");
//				System.out.println(syn.toXML());
			}
	
			out.write("</WORDNET>\n");

		} catch(Exception e){
			e.printStackTrace();
		}
	}
	
	
	public void assignMorphosemantic(String MSfile) {
		 
		try{
			Scanner s = new Scanner(new File(MSfile));
			
			while (s.hasNextLine()) {
				String l = s.nextLine();
				String[] data = l.split("\t");
				
				
			
				if (data.length < 5) continue;
				
				if (!containsKey("eng-30-"+data[4])) continue;
				
				Synset w2 = get("eng-30-"+data[4]);
				
				
				Relation r = new Relation();
				r.ID = "eng-30-"+data[1].trim();
				r.sem = data[2].trim();
				
				w2.relations.add(r);
				put("eng-30-"+data[4], w2);
								
			}
			
			s.close();
			
		} catch(Exception e){
			e.printStackTrace();
		}
		
	}
	
	public ArrayList<Synset> getAllSynsetsContainingLiteral
					(String literal) {
		
		ArrayList<Synset> result = new ArrayList<Synset>(); 
		ArrayList<String> list = index.get(literal);
		
		if (list!=null){
			for (String key : list) {
				result.add(get(key));
			}
		}
		
		return result;
	}
	
	// 1 sister of 2, 1 hypernym of 2, 1 hyponym of 2, 
	// 1 indir-hypernym of 2, 1 indir-hyponym of 2,
	// 1 unrelated to 2
	public String getSynsetHierarchyRelation(String id1, String id2){
		
		String res = "unrelated";
		
		if (getAllHyponymsIDs(get(id1)).contains(id2)) return "hypernym";
		if (getAllHyponymsIDs(get(id2)).contains(id1)) return "hyponym";
		
		if (getAllHyponymsRecursively(get(id1)).contains(id2)) return "indir-hypernym";
		if (getAllHyponymsRecursively(get(id2)).contains(id1)) return "indir-hyponym";
		
		if (get(id1).getHypernymID().equals(get(id2).getHypernymID())) return "sister";

		return res;
	}

	public ArrayList<Synset> getAllSynsetsContainingLiteral(String literal, boolean verified) {
		
		if (!verified) return getAllSynsetsContainingLiteral(literal);
		
		ArrayList<Synset> result = new ArrayList<Synset>(); 
		ArrayList<String> list = index.get(literal);
		
		if (list!=null){
			for (String key : list) {
				Synset s = get(key);
				if (s.definition.contains("MT")) continue;
				if (s.definition.contains("Define me")) continue;
				
				for (Literal l : s.literals) {
					if (l.lemma.equals(literal)
							|| l.word.equals(literal)) {
						if (!l.status.equals("not verified")) result.add(s);
						break;					
					}
				}
			}
		}
		
		return result;
	}
	
	public ArrayList<Synset> getAllSynsetsFromPos(String pos) {
		
		ArrayList<Synset> result = new ArrayList<Synset>(); 
		
		for (String s : keySet()){
			Synset syn = get(s);
			if (syn.pos.toLowerCase().startsWith(pos.toLowerCase())){
				result.add(syn);
			}
		}
		
		return result;
	}


	public ArrayList<Synset> getAllHyponyms(Synset synset) {
		
		ArrayList<Synset> result = new ArrayList<Synset>(); 
				
		ArrayList<String> list = hyponymIndex.get(synset.ID); 
		
		if (list!=null){
			for (String key : list) {
				result.add(get(key));
			}
		}
		
		return result;
	}
	
	public ArrayList<String> getAllHyponymsIDs(Synset synset) {
		
		ArrayList<String> result = new ArrayList<String>(); 
				
		ArrayList<String> list = hyponymIndex.get(synset.ID); 
		
		if (list!=null){
			for (String key : list) {
				result.add(key);
			}
		}
		
		return result;
	}

	public ArrayList<String> getAllHyponymsRecursively(Synset synset) {
		
		ArrayList<String> list = new ArrayList<String>();
		
		if (hyponymIndex.containsKey(synset.ID)){
			list.addAll (hyponymIndex.get(synset.ID)); 
		}
		
		int i = 0;
		while (i<list.size()){
			String hyponym = list.get(i);
			i++;
			
			if (hyponymIndex.containsKey(hyponym)){
				
				ArrayList<String> list1 = hyponymIndex.get( hyponym );
				for (String ss : list1) {
					if (!list.contains(ss))
						list.add(ss);
				}
			}
		}

		return list;
	}
	
	public void extractStats(){
		
		String[] msrelations = {"material","agent","state","event","result",
				"uses","instrument","by-means-of","location","property","undergoer",
				"vehicle","body-part-actor"};

		ArrayList<String> set = new ArrayList<String>();
		 
	     try {
	    	 
	     
		     ArrayList<Synset> nouns = getAllSynsetsFromPos("n");
		     
		     System.out.println(nouns.size());
		     
		     ArrayList<Synset> verbs = getAllSynsetsFromPos("v");
		     
		     System.out.println(verbs.size());
		     
		     int lsin = 0;
		     int lmwe = 0;

		     int lmwen = 0;
		     int lmwev = 0;
		     int lmwea = 0;
		     int lmweadv = 0;
		     int lmweprep = 0;
		     int lmwec = 0;
		     int lmwepro = 0;
		     int lmwepart = 0;
		     
		     int stamp2018 = 0;
		     int hvsyn = 0;
		     int hvl = 0;
		     int alll = 0;
		     int tl = 0;
		     int bgderiv = 0;
		     int semrel = 0;
		     int semrelcheck = 0;
		     
		     int srel = 0;
		     int lrel = 0;

		     int totalsyn = 0;
		     int smwe = 0;
		     
		     int emptydef = 0;
		     int shortdef = 0;
		    		 
		     
		     for (Synset syn : values()) {
		    	 
		    	 boolean sn = false;
		    	 boolean mn = false;
		    	 
		    	 totalsyn++;
		    	 
		    	 srel =srel + syn.relations.size();
		    	 
		    	 alll += syn.literals.size();
		    	 
		    	 if (!syn.definition.contains("Define me") 
		    			 && !syn.definition.startsWith("MT!")
		    			 && !syn.definition.startsWith("MTL!")
		    			 && !syn.status.equals("not verified") ) hvsyn ++;
		    	 else {
		    		 
		    		 continue;
		    	 }
		    	 
		    	 if (syn.definition.length()<10) shortdef++;
		    	 if (syn.definition.trim().length()==0) emptydef++;
		    	 
		    	 if (syn.stamp.contains("2018")
//		    			 && syn.stamp.contains("boby")
		    			 ) stamp2018++;
		    	 
			    	 
mm:		    	 for (Relation r : syn.relations) {
	

					if (r.type.startsWith("has_")) semrelcheck++;
	
		    		 if (r.type.contains("deriv")) {
		    			 
		    			 String key = "";
		    			 if (syn.pos.toLowerCase().startsWith("n")
		    					 && r.ID.endsWith("v")){
		    			  bgderiv++;
		    			  
		    			  key = syn.ID + "\t" + r.ID;
		    			  if (!set.contains(key)) set.add(key);
		    			  
		    			 } else if (syn.pos.toLowerCase().startsWith("v")
		    					 && r.ID.endsWith("n")){
		    			  bgderiv++;
		    			  key = r.ID + "\t" + syn.ID;
		    			  if (!set.contains(key)) set.add(key);
		    			 } else break;
			    		 
			    		 for (Relation r1 : syn.relations) {
			    			 
			    			 for (String ms : msrelations) {
					    		 if (r1.type.contains(ms)
					    				 && r1.type.contains("has")
					    				 && r1.ID.equals(r.ID)) {
					    			 semrel++;
					    			 System.out.println(syn.ID
					    					 +"\t" + r.ID+"_"+r.type
					    					 +"\t" + r1.ID+"_"+r1.type);
					    			 continue mm;
					    		 }
			    			 }
				    	 }
		    		 }
		    	 }
		    	 
		    	 for (Literal l : syn.literals) {
		    		 
		    		 lrel =lrel + l.relations.size();
		    		 
		    		 if (!l.status.contains("not")) hvl++;
		    		 tl ++;
		    		 
		    		 if (l.lemma.contains(" ")) {
		    			 
		    			 lmwe++;
		    			 
		    			 if (!mn) {
		    				 smwe ++;
		    				 mn = true;
		    			 }
		    			 
		    			 if (syn.pos.toLowerCase().startsWith("n")) lmwen ++;
		    			 if (syn.pos.toLowerCase().startsWith("v")) lmwev ++;
		    			 if (syn.pos.toLowerCase().startsWith("a")) lmwea ++;
		    			 if (syn.pos.toLowerCase().startsWith("b")) lmweadv ++;
		    			 if (syn.pos.toLowerCase().startsWith("pron")) lmweprep ++;
//		    			 if (syn.pos.toLowerCase().startsWith("n")) lmwen ++;
//		    			 if (syn.pos.toLowerCase().startsWith("n")) lmwen ++;
//		    			 if (syn.pos.toLowerCase().startsWith("n")) lmwen ++;
		    			 
		    		 } else {
		    			 lsin ++;
		    			
		    		 }
		    		 
		    	 }
		    	 
		    	 
		    	 
		     }
		
		     System.out.println("Total synsets: "+ totalsyn);
		     System.out.println("Total literals: "+ alll);
		     System.out.println("Human verified synsets: "+ hvsyn);
		     System.out.println("Human verified literals: "+ hvl);

		     System.out.println("Unverified literals in human verified synsets: "+ tl);
		     System.out.println("Synsets in 2018: "+ stamp2018);

		     System.out.println("Empty defs: "+ emptydef);
		     System.out.println("Short defs: "+ shortdef);

		     System.out.println("Synsets rels: "+ srel);
		     System.out.println("Literal rels: "+ lrel);
		     System.out.println("BG Deriv: "+ bgderiv+"\t"+set.size());
		     System.out.println("MSRs: "+ semrel+"\t"+semrelcheck);

		     System.out.println("Single literals: "+ lsin);
		     System.out.println("MWE literals: "+ lmwe);
		     System.out.println("Synsets containing MWE: "+ smwe);

		     System.out.println();
		    		 
		     System.out.println("MWE literals N: "+ lmwen);
		     System.out.println("MWE literals V: "+ lmwev);
		     System.out.println("MWE literals A: "+ lmwea);
		     System.out.println("MWE literals D: "+ lmweadv);
		     System.out.println("MWE literals P: "+ lmweprep);

			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

	}

	public ArrayList<String> getAllInstancesRecursively
		(String synsetID) {
		
		
		ArrayList<String> result = new ArrayList<String>(); 
				
		ArrayList<String> listi = getAllHyponymsRecursively(
				get(synsetID)); 
		
		if (listi!=null){
			for (String key : listi) {
				
				if (hyponymInstancesIndex.containsKey( key ))
					result.addAll(
 						hyponymInstancesIndex.get( key ));
			}
		}
		
		return result;
	}

}
